@@ -8,7 +8,8 @@ This module definse the Manager class subclasses by component.BaseComponent
 """
 
 import os
-import time
+from time import sleep
+from warnings import warn
 from itertools import chain
 from types import TupleType
 from threading import Thread
@@ -47,6 +48,8 @@ from values import Value
 from events import Started, Stopped, Signal
 from events import Error, Success, Failure, Filter, Start, End
 
+TIMEOUT = 1.0 # 1s timeout when no tick functions to process
+
 class Manager(object):
     """Manager
 
@@ -470,7 +473,12 @@ class Manager(object):
         if signal in [SIGINT, SIGTERM]:
             self.stop()
 
-    def start(self, sleep=0, log=True, link=None, process=False):
+    def start(self, sleep=None, log=True, link=None, process=False):
+        if sleep:
+            warn(DeprecationWarning(
+                "sleep will be deprecated in a future release. " +
+                "Set circuits.core.TIMEOUT instead."))
+
         group = None
         target = self.run
         name = self.__class__.__name__
@@ -513,13 +521,22 @@ class Manager(object):
         if self._ticks:
             try:
                 [f() for f in self._ticks.copy()]
+            except (KeyboardInterrupt, SystemExit):
+                raise
             except:
                 etype, evalue, etraceback = _exc_info()
                 self.fire(Error(etype, evalue, format_tb(etraceback)))
         if self:
             self._flush()
+        else:
+            sleep(TIMEOUT) # Nothing to do - Let's not tie up the CUP
+
+    def run(self, sleep=None, log=True, __mode=None, __socket=None):
+        if sleep:
+            warn(DeprecationWarning(
+                "sleep will be deprecated in a future release. " +
+                "Set circuits.core.TIMEOUT instead."))
 
-    def run(self, sleep=0, log=True, __mode=None, __socket=None):
         if not __mode == "T" and current_thread().getName() == "MainThread":
             if os.name == "posix":
                 _registerSignalHandler(SIGHUP, self._signalHandler)
@@ -548,10 +565,9 @@ class Manager(object):
         try:
             while self or self.running:
                 self.tick()
-                if sleep:
-                    time.sleep(sleep)
         finally:
             if __socket is not None:
                 while bridge or manager: pass
                 manager.stop()
                 bridge.stop()
+            self.stop()
