@@ -14,11 +14,21 @@ from _socket import socket as SocketType
 from collections import defaultdict, deque
 from socket import gethostname, gethostbyname
 
-from circuits.net.pollers import Select as DefaultPoller
+try:
+    import ssl
+    import ssl as ssl_ # Doh, otherwise local variables overshadow the module
+    HAS_SSL = 1
+except ImportError:
+    import warnings
+    warnings.warn("No SSL support available. Using python-2.5 ? Try isntalling the ssl module: http://pypi.python.org/pypi/ssl/")
+    HAS_SSL = 0
+
+from circuits.core.utils import findcmp
 from circuits.core import handler, Event, Component
+from circuits.core.pollers import _Poller, Select as DefaultPoller
 
 BUFSIZE = 4096 # 4KB Buffer
-BACKLOG = 128  # 128 Concurrent Connections
+BACKLOG = 5000 #  5K Concurrent Connections
 
 ###
 ### Event Objects
@@ -33,11 +43,11 @@ class Connect(Event):
 
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: (host, port) Server: (sock, host, port)
-    @type  args: tuple
+    :param args:  Client: (host, port) Server: (sock, host, port)
+    :type  args: tuple
 
-    @param kwargs: Client: (ssl)
-    @type  kwargs: dict
+    :param kwargs: Client: (ssl)
+    :type  kwargs: dict
     """
 
     def __init__(self, *args, **kwargs):
@@ -53,8 +63,8 @@ class Disconnect(Event):
 
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: () Server: (sock)
-    @type  tuple: tuple
+    :param args:  Client: () Server: (sock)
+    :type  tuple: tuple
     """
 
     def __init__(self, *args):
@@ -69,11 +79,11 @@ class Connected(Event):
 
     @note: This event is for Client Components.
 
-    @param host: The hostname connected to.
-    @type  str:  str
+    :param host: The hostname connected to.
+    :type  str:  str
 
-    @param port: The port connected to
-    @type  int:  int
+    :param port: The port connected to
+    :type  int:  int
     """
 
     def __init__(self, host, port):
@@ -101,8 +111,8 @@ class Read(Event):
 
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: (data) Server: (sock, data)
-    @type  tuple: tuple
+    :param args:  Client: (data) Server: (sock, data)
+    :type  tuple: tuple
     """
 
     def __init__(self, *args):
@@ -117,8 +127,8 @@ class Error(Event):
 
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: (error) Server: (sock, error)
-    @type  tuple: tuple
+    :param args:  Client: (error) Server: (sock, error)
+    :type  tuple: tuple
     """
 
     def __init__(self, *args):
@@ -135,8 +145,8 @@ class Write(Event):
     @note: This event is never sent, it is used to send data.
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: (data) Server: (sock, data)
-    @type  tuple: tuple
+    :param args:  Client: (data) Server: (sock, data)
+    :type  tuple: tuple
     """
 
     def __init__(self, *args):
@@ -154,8 +164,8 @@ class Close(Event):
     @note: This event is never sent, it is used to close.
     @note: This event is used for both Client and Server Components.
 
-    @param args:  Client: () Server: (sock)
-    @type  tuple: tuple
+    :param args:  Client: () Server: (sock)
+    :type  tuple: tuple
     """
 
     def __init__(self, *args):
@@ -163,6 +173,26 @@ class Close(Event):
 
         super(Close, self).__init__(*args)
 
+class Ready(Event):
+    """Ready Event
+
+    This Event is used to notify the rest of the system that the underlying
+    Client or Server Component is ready to begin processing connections or
+    incoming/outgoing data. (This is triggered as a direct result of having
+    the capability to support multiple client/server components with a snigle
+    poller component instnace in a system).
+
+    @note: This event is used for both Client and Server Components.
+
+    :param component:  The Client/Server Component that is ready.
+    :type  tuple: Component (Client/Server)
+    """
+
+    def __init__(self, component):
+        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
+
+        super(Ready, self).__init__(component)
+
 ###
 ### Components
 ###
@@ -171,8 +201,11 @@ class Client(Component):
 
     channel = "client"
 
-    def __init__(self, bind=None, channel=channel, **kwargs):
-        super(Client, self).__init__(channel=channel, **kwargs)
+    def __init__(self, bind=None, **kwargs):
+        channel = kwargs.get("channel", self.__class__.channel)
+        super(Client, self).__init__(channel=channel)
+
+        self.encoding = kwargs.get("encoding", "utf-8")
 
         if type(bind) is int:
             self.bind = (gethostbyname(gethostname()), bind)
@@ -192,10 +225,8 @@ class Client(Component):
 
         self._bufsize = kwargs.get("bufsize", BUFSIZE)
 
-        Poller = kwargs.get("poller", DefaultPoller)
-
-        self._poller = Poller()
-        self._poller.register(self)
+        self._poller = None
+        self._PollerComponent = kwargs.get("poller", DefaultPoller)
 
         self._sock = None
         self._sslsock = None
@@ -207,6 +238,30 @@ class Client(Component):
     def connected(self):
         return getattr(self, "_connected", None)
 
+    @handler("registered", target="*")
+    def _on_registered(self, component, manager):
+        if self._poller is None:
+            if isinstance(component, _Poller):
+                self._poller = component
+                self.push(Ready(self), "ready", self.channel)
+            else:
+                component = findcmp(self.root, _Poller, subclass=False)
+                if component is not None:
+                    self._poller = component
+                    self.push(Ready(self), "ready", self.channel)
+                else:
+                    self._poller = self._PollerComponent()
+                    self._poller.register(self.root)
+                    self.push(Ready(self), "ready", self.channel)
+
+    @handler("started", filter=True, target="*")
+    def _on_started(self, component, mode):
+        if self._poller is None:
+            self._poller = self._PollerComponent()
+            self._poller.register(self.root)
+            self.push(Ready(self), "ready", self.channel)
+            return True
+
     def _close(self):
         self._poller.discard(self._sock)
 
@@ -247,6 +302,9 @@ class Client(Component):
 
     def _write(self, data):
         try:
+            if type(data) is unicode:
+                data = data.encode(self.encoding)
+
             if self.ssl and self._sslsock:
                 bytes = self._sslsock.write(data)
             else:
@@ -262,7 +320,7 @@ class Client(Component):
 
     def write(self, data):
         if not self._poller.isWriting(self._sock):
-            self._poller.addWriter(self._sock)
+            self._poller.addWriter(self, self._sock)
         self._buffer.append(data)
 
     @handler("_disconnect", filter=True)
@@ -321,11 +379,11 @@ class TCPClient(Client):
 
         self._connected = True
 
-        self._poller.addReader(self._sock)
+        self._poller.addReader(self, self._sock)
 
         if self.ssl:
             self._sslsock = socket.ssl(self._sock)
-        
+
         self.push(Connected(host, port), "connected", self.channel)
 
 class UNIXClient(Client):
@@ -342,6 +400,11 @@ class UNIXClient(Client):
 
         self._sock.setblocking(False)
 
+    @handler("registered", target="*")
+    def _on_registered(self, component, manager):
+        if self._poller is not None and self._connected:
+            self._poller.addReader(self, self._sock)
+
     def connect(self, path, ssl=False):
         self.path = path
         self.ssl = ssl
@@ -362,11 +425,11 @@ class UNIXClient(Client):
 
         self._connected = True
 
-        self._poller.addReader(self._sock)
+        self._poller.addReader(self, self._sock)
 
         if self.ssl:
             self._sslsock = socket.ssl(self._sock)
-        
+
         self.push(Connected(gethostname(), path), "connected", self.channel)
 
 
@@ -374,9 +437,12 @@ class Server(Component):
 
     channel = "server"
 
-    def __init__(self, bind, ssl=False, channel=channel, **kwargs):
+    def __init__(self, bind, ssl=False, **kwargs):
+        channel = kwargs.get("channel", self.__class__.channel)
         super(Server, self).__init__(channel=channel)
 
+        self.encoding = kwargs.get("encoding", "utf-8")
+
         if type(bind) is int:
             self.bind = (gethostbyname(gethostname()), bind)
         elif type(bind) is str and ":" in bind:
@@ -387,14 +453,18 @@ class Server(Component):
             self.bind = bind
 
         self.ssl = ssl
+        if self.ssl:
+            self.certfile = kwargs.get("certfile", None)
+            self.keyfile = kwargs.get("keyfile", None)
+            self.cert_reqs = kwargs.get("cert_reqs", ssl_.CERT_NONE)
+            self.ssl_version = kwargs.get("ssl_version", ssl_.PROTOCOL_SSLv23)
+            self.ca_certs = kwargs.get("ca_certs", None)
 
         self._bufsize = kwargs.get("bufsize", BUFSIZE)
         self._backlog = kwargs.get("backlog", BACKLOG)
 
-        Poller = kwargs.get("poller", DefaultPoller)
-
-        self._poller = Poller()
-        self._poller.register(self)
+        self._poller = None
+        self._PollerComponent = kwargs.get("poller", DefaultPoller)
 
         self._sock = None
         self._buffers = defaultdict(deque)
@@ -413,8 +483,36 @@ class Server(Component):
     def port(self):
         return self.bind[1] if hasattr(self, "bind") else None
 
+    @handler("registered", target="*")
+    def _on_registered(self, component, manager):
+        if self._poller is None:
+            if isinstance(component, _Poller):
+                self._poller = component
+                self._poller.addReader(self, self._sock)
+                self.push(Ready(self), "ready", self.channel)
+            else:
+                component = findcmp(self.root, _Poller, subclass=False)
+                if component is not None:
+                    self._poller = component
+                    self._poller.addReader(self, self._sock)
+                    self.push(Ready(self), "ready", self.channel)
+                else:
+                    self._poller = self._PollerComponent()
+                    self._poller.register(self.root)
+                    self._poller.addReader(self, self._sock)
+                    self.push(Ready(self), "ready", self.channel)
+
+    @handler("started", filter=True, target="*")
+    def _on_started(self, component, mode):
+        if self._poller is None:
+            self._poller = self._PollerComponent()
+            self._poller.register(self.root)
+            self._poller.addReader(self, self._sock)
+            self.push(Ready(self), "ready", self.channel)
+            return True
+
     def _close(self, sock):
-        if sock not in self._clients:
+        if not sock == self._sock and sock not in self._clients:
             return
 
         self._poller.discard(sock)
@@ -434,10 +532,17 @@ class Server(Component):
         self.push(Disconnect(sock), "disconnect", self.channel)
 
     def close(self, sock=None):
-        if not self._buffers[sock]:
-            self._close(sock)
-        elif sock not in self._closeq:
-            self._closeq.append(sock)
+        if sock is None:
+            socks = [self._sock]
+            socks.extend(self._clients[:])
+        else:
+            socks = [sock]
+
+        for sock in socks:
+            if not self._buffers[sock]:
+                self._close(sock)
+            elif sock not in self._closeq:
+                self._closeq.append(sock)
 
     def _read(self, sock):
         if sock not in self._clients:
@@ -461,6 +566,9 @@ class Server(Component):
             return
 
         try:
+            if type(data) is unicode:
+                data = data.encode(self.encoding)
+
             bytes = sock.send(data)
             if bytes < len(data):
                 self._buffers[sock].appendleft(data[bytes:])
@@ -472,7 +580,7 @@ class Server(Component):
 
     def write(self, sock, data):
         if not self._poller.isWriting(sock):
-            self._poller.addWriter(sock)
+            self._poller.addWriter(self, sock)
         self._buffers[sock].append(data)
 
     def broadcast(self, data):
@@ -482,6 +590,18 @@ class Server(Component):
     def _accept(self):
         try:
             newsock, host = self._sock.accept()
+            if self.ssl and HAS_SSL:
+                newsock = ssl.wrap_socket(newsock,
+                    server_side=True,
+                    certfile=self.certfile,
+                    keyfile=self.keyfile,
+                    cert_reqs=self.cert_reqs,
+                    ssl_version=self.ssl_version,
+                    ca_certs=self.ca_certs)
+
+        except ssl_.SSLError, e:
+            raise
+
         except socket.error, e:
             if e[0] in (EWOULDBLOCK, EAGAIN):
                 return
@@ -509,7 +629,7 @@ class Server(Component):
                 raise
 
         newsock.setblocking(False)
-        self._poller.addReader(newsock)
+        self._poller.addReader(self, newsock)
         self._clients.append(newsock)
         self.push(Connect(newsock, *host), "connect", self.channel)
 
@@ -553,12 +673,9 @@ class TCPServer(Server):
         self._sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         self._sock.setblocking(False)
         if not bound:
-            print type(self.bind), repr(self.bind)
             self._sock.bind(self.bind)
         self._sock.listen(self._backlog)
 
-        self._poller.addReader(self._sock)
-
 class UNIXServer(Server):
 
     def __init__(self, bind, ssl=False, **kwargs):
@@ -585,8 +702,6 @@ class UNIXServer(Server):
         self._sock.setblocking(False)
         self._sock.listen(self._backlog)
 
-        self._poller.addReader(self._sock)
-
 class UDPServer(Server):
 
     def __init__(self, bind, ssl=False, **kwargs):
@@ -598,16 +713,13 @@ class UDPServer(Server):
             self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             self._sock.bind(self.bind)
 
-        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
         self._sock.setblocking(False)
 
-        self._poller.addReader(self._sock)
+    def _close(self, sock):
+        self._poller.discard(sock)
 
-    def _close(self):
-        self._poller.discard(self._sock)
-
-        if self._sock in self._buffers:
+        if sock in self._buffers:
             del self._buffers[sock]
 
         try:
@@ -628,14 +740,17 @@ class UDPServer(Server):
             data, address = self._sock.recvfrom(self._bufsize)
             if data:
                 self.push(Read(address, data), "read", self.channel)
-            else:
-                self.close(self._sock)
         except socket.error, e:
+            if e[0] in (EWOULDBLOCK, EAGAIN):
+                return
             self.push(Error(self._sock, e), "error", self.channel)
             self._close(self._sock)
 
     def _write(self, address, data):
         try:
+            if type(data) is unicode:
+                data = data.encode(self.encoding)
+
             bytes = self._sock.sendto(data, address)
             if bytes < len(data):
                 self._buffers[self._sock].appendleft(data[bytes:])
@@ -648,7 +763,7 @@ class UDPServer(Server):
     @handler("write", override=True)
     def write(self, address, data):
         if not self._poller.isWriting(self._sock):
-            self._poller.addWriter(self._sock)
+            self._poller.addWriter(self, self._sock)
         self._buffers[self._sock].append((address, data))
 
     def broadcast(self, data, port):
@@ -677,7 +792,7 @@ class UDPServer(Server):
 
 UDPClient = UDPServer
 
-def Pipe(channels=("pipe.a", "pipe.b"), **kwargs):
+def Pipe(channels=("pipe", "pipe"), **kwargs):
     """Create a new full duplex Pipe
 
     Returns a pair of UNIXClient instances connected on either side of
@@ -688,7 +803,5 @@ def Pipe(channels=("pipe.a", "pipe.b"), **kwargs):
     a = UNIXClient(s1, channel=channels[0], **kwargs)
     b = UNIXClient(s2, channel=channels[1], **kwargs)
     a._connected = True
-    a._poller.addReader(a._sock)
     b._connected = True
-    b._poller.addReader(b._sock)
     return a, b
